<?php

namespace backend\modules\places\models;

use backend\modules\city\models\City;
use backend\modules\district\models\District;
use backend\modules\subdistrict\models\Subdistrict;
use bedezign\yii2\audit\AuditTrailBehavior;
use common\models\User;
use kartik\helpers\Html;
use libphonenumber\NumberParseException;
use libphonenumber\PhoneNumberFormat;
use libphonenumber\PhoneNumberUtil;
use yeesoft\behaviors\MultilingualBehavior;
use yeesoft\db\ActiveRecord;
use Yii;
use yii\behaviors\BlameableBehavior;
use yii\behaviors\SluggableBehavior;
use yii\behaviors\TimestampBehavior;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "places".
 *
 * @property int $id
 * @property string $slug
 * @property int $kinz_id
 * @property int $city_id
 * @property int $district_id
 * @property int $sub_district_id
 * @property int $status 0-pending,1-published
 * @property string $phone
 * @property string $website
 * @property string $latitude
 * @property string $longitude
 * @property int $created_at
 * @property int $updated_at
 * @property int $created_by
 * @property int $updated_by
 * @property string $thumbnail
 *
 */
class Places extends ActiveRecord
{

    public $classifications;
    public $file;

    const STATUS_PENDING = 0;
    const STATUS_PUBLISHED = 1;


    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'places';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['title', 'city_id'], 'required'],
            [['status', 'id', 'kinz_id', 'city_id', 'district_id', 'sub_district_id', 'created_at', 'updated_at', 'created_by', 'updated_by'], 'integer'],
            [['latitude', 'longitude'], 'number'],
            [['phone', 'website', 'slug'], 'string', 'max' => 255],
            [['description'], 'string'],
            [['thumbnail'], 'string', 'max' => 255],
        ];
    }


    /**
     * @inheritdoc
     */
    public function init()
    {
        parent::init();

        $this->on(self::EVENT_AFTER_INSERT, [$this, 'saveImages']);
        $this->on(self::EVENT_AFTER_UPDATE, [$this, 'saveImages']);
        $this->on(self::EVENT_AFTER_INSERT, [$this, 'saveClassifications']);
        $this->on(self::EVENT_AFTER_UPDATE, [$this, 'saveClassifications']);
        $this->on(self::EVENT_AFTER_INSERT, [$this, 'saveAccessibilities']);
        $this->on(self::EVENT_AFTER_UPDATE, [$this, 'saveAccessibilities']);
    }


    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [

            'AuditTrail' => [
                'class' => AuditTrailBehavior::className(),
            ],
            'Timestamp' => [
                'class' => TimestampBehavior::className(),
            ],
            'Blameable' => [
                'class' => BlameableBehavior::className(),
            ],
            'sluggable' => [
                'class' => SluggableBehavior::className(),
                'ensureUnique' => true,
                'attribute' => ['title'],
            ],
            'multilingual' => [
                'class' => MultilingualBehavior::className(),
                'langForeignKey' => 'place_id',
                'tableName' => "{{%places_lang}}",
                'attributes' => [
                    'title',
                    'description'
                ]
            ],
        ];
    }


    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('site', 'ID'),
            'kinz_id' => Yii::t('site', 'Kinz ID'),
            'city_id' => Yii::t('site', 'City'),
            'district_id' => Yii::t('site', 'District'),
            'sub_district_id' => Yii::t('site', 'District'),
            'phone' => Yii::t('site', 'Phone'),
            'website' => Yii::t('site', 'Website'),
            'latitude' => Yii::t('site', 'Latitude'),
            'longitude' => Yii::t('site', 'Longitude'),
            'created_at' => Yii::t('site', 'Created At'),
            'updated_at' => Yii::t('site', 'Updated At'),
            'created_by' => Yii::t('site', 'Created By'),
            'updated_by' => Yii::t('site', 'Updated By'),
            'status' => Yii::t('site', 'Status'),
            'thumbnail' => Yii::t('site', 'Thumbnail'),
            'title' => Yii::t('site', 'Name of Place'),
            'subsector_id' => Yii::t('site', 'Type of Place'),
            'description' => Yii::t('site', 'Description'),

        ];
    }


    public function getAccessibilityValues()
    {

        return ArrayHelper::map(PlaceAccessibility::find()->where(['place_id' => $this->id])->asArray()->all(), 'accessibility_id', 'accessibility_id');
    }


    /**
     * {@inheritdoc}
     * @return PlacesQuery the active query used by this AR class.
     */
    public static function find()
    {
        return new PlacesQuery(get_called_class());
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getImages()
    {
        return $this->hasMany(PlaceImages::className(), ['place_id' => 'id']);
    }
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getAccessibilites()
    {
        return $this->hasMany(Accessibility::className(), ['id' => 'accessibility_id'])->viaTable('place_accessibility', ['place_id' => 'id']);
    }


    /**
     * @return \yii\db\ActiveQuery
     */
    public function getFilters()
    {
        return $this->hasMany(PlaceFilter::className(), ['place_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSubsectors()
    {
        return $this->hasMany(PlaceSubsector::className(), ['place_id' => 'id']);
    }


    public function getCity()
    {
        return $this->hasOne(City::className(), ['id' => 'city_id']);
    }


    public function getDistrict()
    {
        return $this->hasOne(District::className(), ['id' => 'district_id']);
    }

    public function getSubDistrict()
    {
        return $this->hasOne(Subdistrict::className(), ['id' => 'sub_district_id']);
    }


    public function getDistrictName()
    {
        $district = $this->district;

        return ($district ? $district->title : '');
    }
    public function getSubDistrictName()
    {
        $subdistrict = $this->subDistrict;

        return ($subdistrict ? $subdistrict->title : '');
    }

    public function getAuthor()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }
    public function getPlaceAggregateRating()
    {
        return $this->hasOne(PlaceAggregateRating::className(), ['model_id' => 'id']);
    }

    public function getPublishedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->published_at);
    }

    public function getCreatedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->created_at);
    }

    public function getUpdatedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->updated_at);
    }

    public function getPublishedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->published_at);
    }

    public function getCreatedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->created_at);
    }

    public function getUpdatedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->updated_at);
    }

    public function getPublishedDatetime()
    {
        return "{$this->publishedDate} {$this->publishedTime}";
    }

    public function getCreatedDatetime()
    {
        return "{$this->createdDate} {$this->createdTime}";
    }

    public function getUpdatedDatetime()
    {
        return "{$this->updatedDate} {$this->updatedTime}";
    }

    public static function populateFromSolr($doc){

        return Places::findOne($doc['id']);
    }
    /**
     *
     * @inheritdoc
     */
    public static function getFullAccessPermission()
    {
        return 'fullPageAccess';
    }

    /**
     *
     * @inheritdoc
     */
    public static function getOwnerField()
    {
        return 'created_by';
    }

    public function getThumbnail($options = ['class' => 'thumbnail pull-left', 'style' => 'width: 240px'])
    {
        if (!empty($this->thumbnail)) {
            return Html::img($this->thumbnail, $options);
        }

        return Html::img("/images/blueBG.png", $options);
    }

    /**
     *
     * @inheritdoc
     */
    public function getFormattedPhone($format = PhoneNumberFormat::INTERNATIONAL)
    {
        $phoneUtil = PhoneNumberUtil::getInstance();
        $phone = $this->phone;
        try {
            $numberProto = $phoneUtil->parse($this->phone, "JO");
            $phone = $phoneUtil->format($numberProto, $format);
        } catch (NumberParseException $e) {
        }

        return $phone;

    }

    /**
     *
     * @inheritdoc
     */
    public function getGoogleMapDirection()
    {
        return "https://maps.google.com?saddr=Current+Location&daddr={$this->latitude},{$this->longitude}";

    }

    /**
     *
     * @inheritdoc
     */
    public function getWebsiteWithHttp()
    {
        $url = $this->website;
        if (!preg_match("~^(?:f|ht)tps?://~i", $this->website)) {
            $url = "http://" . $this->website;
        }
        return $url;
    }

    public function isVarified(){
        return PlaceAccessibility::find()->where(['place_id' => $this->id, 'accessibility_id' => 20])->count();
    }

    /**
     * Handle save tags event of the owner.
     */
    public function saveImages()
    {
        /** @var Places $owner */
        $owner = $this->owner;

        $post = Yii::$app->getRequest()->post('Places');
        $images = (isset($post['images'])) ? $post['images'] : null;
        if (is_array($images)) {
            foreach ($images as $img) {
                PlaceImages::deleteAll(['place_id' => $owner->id, 'path' => $img]);
                $linkImage = new PlaceImages();
                $linkImage->path = $img;
                $linkImage->place_id = $owner->id;
                $linkImage->save();
            }
        }
    }

    /**
     * Handle save tags event of the owner.
     */
    public function saveAccessibilities()
    {
        /** @var Places $owner */
        $owner = $this->owner;

        $accessibilityValues = @Yii::$app->getRequest()->post('Places')['accessibilityValues'];

        PlaceAccessibility::deleteAll(['place_id' => $owner->id]);

        if (is_array($accessibilityValues)) {
            foreach ($accessibilityValues as $accessibility) {
                $linkImage = new PlaceAccessibility();
                $linkImage->accessibility_id = $accessibility;
                $linkImage->place_id = $owner->id;
                $linkImage->save();
            }
        }

        // Recalculate Place Rating specially after change place Accessibility
        PlaceRating::updateRating($owner->id);
    }

    /**
     * Handle save tags event of the owner.
     */
    public function saveClassifications()
    {
        /** @var Places $owner */
        $owner = $this->owner;

        $classifications = Yii::$app->getRequest()->post('ClassificationForm');
        if (is_array($classifications)) {
            PlaceSubsector::deleteAll(['place_id' => $owner->id]);
            PlaceFilter::deleteAll(['place_id' => $owner->id]);
            foreach ($classifications as $classification) {
                $linkImage = new PlaceSubsector();
                $linkImage->subsector_id = $classification['subsector'];
                $linkImage->place_id = $owner->id;
                $linkImage->save();

                $linkImage = new PlaceFilter();
                $linkImage->filter_id = $classification['filter'];
                $linkImage->place_id = $owner->id;
                $linkImage->save();
            }
        }

    }

    public function save($runValidation = true, $attributeNames = null)
    {
        \Yii::$app->cache->flush();
        return parent::save($runValidation, $attributeNames);
    }

//    public function createThumbs(array $routes, array $presets)
//    {
//        $thumbs = [];
//        $basePath = $basePath = Yii::getAlias($routes['basePath']);
//        $originalFile = pathinfo($this->url);
//        $dirname = $originalFile['dirname'];
//        $filename = $originalFile['filename'];
//        $extension = $originalFile['extension'];
//
//        Imagine::$driver = [Imagine::DRIVER_GD2, Imagine::DRIVER_GMAGICK, Imagine::DRIVER_IMAGICK];
//
//        foreach ($presets as $alias => $preset) {
//            $width = $preset['size'][0];
//            $height = $preset['size'][1];
//
//            $thumbUrl = "$dirname/$filename-{$width}x{$height}.$extension";
//
//            Imagine::thumbnail("$basePath/{$this->url}", $width, $height)->save("$basePath/$thumbUrl");
//
//            $thumbs[$alias] = $thumbUrl;
//        }
//
//        $this->thumbs = serialize($thumbs);
//        $this->detachBehavior('timestamp');
//
//        // create default thumbnail
//        $this->createDefaultThumb($routes);
//
//        return $this->save();
//    }
}
