<?php
namespace common\models;

use common\helpers\JPA_WS\JPA_Profile;
use common\helpers\JPA_WS\JPA_WS;
use yeesoft\helpers\Html;
use Yii; 
use yii\base\NotSupportedException;
use yii\behaviors\TimestampBehavior;
use yii\db\ActiveRecord;
use yii\helpers\ArrayHelper;
use yii\web\IdentityInterface;

/**
 * User model
 *
 * @property integer $id
 * @property string $username
 * @property string $password_hash
 * @property string $password_reset_token
 * @property string $verification_token
 * @property string $email
 * @property string $source
 * @property string $auth_key
 * @property integer $status
 * @property integer $created_at
 * @property integer $email_verified_at
 * @property integer $updated_at
 * @property string $password write-only password
 * @property string $name
 * @property int $type
 * @property int $is_document_approved
 * @property string $document
 * @property string $profile_image
 * @property integer $phone
 * @property string $address
 * @property string $city
 * @property integer $code
 * @property string $country
 * @property string $about_me
 */
class Users extends ActiveRecord implements IdentityInterface
// class Users extends \yeesoft\models\User
{
    const STATUS_DELETED = 0;
    const STATUS_INACTIVE = 9;
    const STATUS_ACTIVE = 10;

    const SOURCE_WEB = 'web';
    const SOURCE_FACEBOOK = 'facebook';
    const SOURCE_GOOLE = 'google';

    const OWNER = 1;
    const DEALER = 2;

    const REVIEW = 0;
    const APPROVED = 1;
    const REJECTED = 2;

    public $old_password;
    public $new_password;
    public $confirm_password;


    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return '{{%users}}';
    }
    /**
     * {@inheritdoc}
     */
    public function rules()
    {

        return [
            ['status', 'default', 'value' => self::STATUS_INACTIVE],
            ['status', 'in', 'range' => [self::STATUS_ACTIVE, self::STATUS_INACTIVE, self::STATUS_DELETED]],

            ['email', 'trim'],
            ['email', 'required'],
            ['email', 'email'],
            [['email', 'name'],'string', 'max' => 255],
            ['email', 'unique', 'targetClass' => '\common\models\Users', 'message' => Yii::t('auto', 'This email address has already been taken.')],

            [['type','is_document_approved', 'owner_id'],'integer'],
            [['owner_id'], 'safe'],
            [['email_verified_at', 'source'], 'safe'],
            [['name','document', 'profile_image', 'phone', 'address', 'city', 'code', 'country', 'about_me'], 'safe'],
            [['city', 'country'], 'string', 'max' => 125],
            [['code'],'integer'],
            [['phone'],'number'],
            [['about_me'],'string', 'max' => '550'],
            ['password', 'required'],
            ['name', 'required'],
            [['email','password','name','image'],'required','on' => 'car_dealer'],
            [['email','password','name'],'required','on' => 'car_owner'],
            ['password', 'string', 'min' => Yii::$app->params['user.passwordMinLength']],
            ['old_password', 'validatePassword', 'on' => 'change-password'],
            ['new_password', 'samePassword', 'on' => 'change-password'],
            ['new_password', 'match',
                //'pattern' => '/^(?=.*[0-9])(?=.*[A-Z])([a-zA-Z0-9]+)$/',
                'pattern' => '/^(?=.*[A-Za-z])(?=.*\d)(?=.*[@$!%*#?&])[A-Za-z\d@$!%*#?&]{6,}$/',
                'message' => Yii::t('auto','Password must have minimum six characters, at least one letter, one number and one special character'),
            ],
            [['old_password', 'new_password', 'confirm_password'],'required', 'on' => 'change-password'],
            ['confirm_password', 'compare', 'compareAttribute'=>'new_password', 'message'=>"Passwords don't match" , 'on' => 'change-password']
        ];
    }

    /**
     * getUsersList
     *
     * @return array
     */
    public static function getUsersList()
    {
        $users = static::find()->select(['id', 'name'])->asArray()->all();
        return ArrayHelper::map($users, 'id', 'name');
    }


    /**
     * {@inheritdoc}
     */
    public static function findIdentity($id)
    {
        return static::findOne(['id' => $id, 'status' => self::STATUS_ACTIVE]);
    }

    /**
     * {@inheritdoc}
     */
    public static function findIdentityByAccessToken($token, $type = null)
    {
        throw new NotSupportedException('"findIdentityByAccessToken" is not implemented.');
    }

    /**
     * Finds user by username
     *
     * @param string $username
     * @return static|null
     */
    public static function findByUsername($username)
    {
        return static::findOne(['name' => $username , 'status' => self::STATUS_ACTIVE]);
    }
    /**
     * Finds user by usernamek
     *
     * @param string $email
     * @return static|null
     */
    public static function findByEmail($email)
    {
        return static::findOne(['email' => $email, 'status' => self::STATUS_ACTIVE]);
    }

    /**
     * Finds user by password reset token
     *
     * @param string $token password reset token
     * @return static|null
     */
    public static function findByPasswordResetToken($token)
    {
        if (!static::isPasswordResetTokenValid($token)) {
            return null;
        }

        return static::findOne([
            'password_reset_token' => $token,
            'status' => self::STATUS_ACTIVE,
        ]);
    }

    /**
     * Finds user by verification email token
     *
     * @param string $token verify email token
     * @return static|null
     */
    public static function findByVerificationToken($token) {
        return static::findOne([
            'verification_token' => $token,
            'status' => self::STATUS_INACTIVE
        ]);
    }

    /**
     * Finds out if password reset token is valid
     *
     * @param string $token password reset token
     * @return bool
     */
    public static function isPasswordResetTokenValid($token)
    {
        if (empty($token)) {
            return false;
        }

        $timestamp = (int) substr($token, strrpos($token, '_') + 1);
        $expire = Yii::$app->params['user.passwordResetTokenExpire'];
        return $timestamp + $expire >= time();
    }

    /**
     * {@inheritdoc}
     */
    public function getId()
    {
        return $this->getPrimaryKey();
    }

    /**
     * {@inheritdoc}
     */
    public function getAuthKey()
    {
        return $this->auth_key;
    }

    /**
     * {@inheritdoc}
     */
    public function validateAuthKey($authKey)
    {
        return $this->getAuthKey() === $authKey;
    }

    /**
     * Validates password
     *
     * @param string $password password to validate
     * @return bool if password provided is valid for current user
     */
    public function validatePassword($password)
    {
        return Yii::$app->security->validatePassword($password, $this->password);
    }

    /**
     * Generates password hash from password and sets it to the model
     *
     * @param string $password
     */
    public function setPassword($password)
    {
        $this->password = Yii::$app->security->generatePasswordHash($password);
    }

    /**
     * Generates "remember me" authentication key
     */
    public function generateAuthKey()
    {
        $this->auth_key = Yii::$app->security->generateRandomString();
    }

    /**
     * Generates new password reset token
     */
    public function generatePasswordResetToken()
    {
        $this->password_reset_token = Yii::$app->security->generateRandomString() . '_' . time();
    }

    /**
     * Generates new token for email verification
     */
    public function generateEmailVerificationToken()
    {
        $this->verification_token = Yii::$app->security->generateRandomString() . '_' . time();
    }

    /**
     * Removes password reset token
     */

    public function removePasswordResetToken()
    {
        $this->password_reset_token = null;
    }

    public function getUser()
    {
        return $this->hasMany(\common\models\FavoriteVehicle::class,['user_id','id']);
    }

    public function getUsername()
    {
        return $this->hasMany(VehicleBids::class,['user_id','id']);
    }

    public function samePassword(){
        $password_exists = Yii::$app->security->validatePassword($this->new_password,   Yii::$app->user->identity->password);

        if($password_exists){
            $this->addError('new_password','New password same as old password');
        }
    }

    public function getAddress() {
        $address = [];
        $address[] = $this->address;
        $address[] = $this->city;
        $address[] = $this->code;
        $address[] = $this->country;

        $address = array_filter($address);

        if(count($address)>=1){
            return implode(', ',$address);
        }
        return '-';
    }

    public function attributeLabels()
    {
        if(!Yii::$app->user->isGuest && Yii::$app->user->identity->type==self::DEALER){
            $name = Yii::t('auto','Business Name');
        }else{
            $name = Yii::t('auto','Name');
        }
        return ArrayHelper::merge([
            'name' => $name
        ],parent::attributeLabels());
    }

    public static function personalLabel()
    {
        if(!Yii::$app->user->isGuest && Yii::$app->user->identity->type==self::DEALER){
            $name = Yii::t('auto','Business Details');
        }else{
            $name = Yii::t('auto','Personal Details');
        }
        return $name;
    }

    public function getProfile()
    {

        if(Yii::$app->id != 'JPA-API' )
            $userProfile = Yii::$app->session->get('__currentUserProfile');

        if (!$userProfile) {
            $profile = new JPA_Profile();

            $jsonResponse = JPA_WS::getProfile($this->username);
            $profile->load(['JPA_Profile' => $jsonResponse], 'JPA_Profile');
            $userProfile = $profile;

            if(Yii::$app->id != 'JPA-API')
                Yii::$app->session->set('__currentUserProfile', $profile);
        }

        return $userProfile;
    }

    /**
     * Sends an email to the specified email address using the information collected by this model.
     *
     *
     * @return bool whether the email was sent
     */
    public function sendEmailToNotifyDealerRegistration()
    {
        return Yii::$app->mailer
            ->compose(
                ['html' => 'new-dealer-registration-html', 'text' => 'new-dealer-registration-text'],
                ['model' => $this]
            )
            ->setTo(Yii::$app->params['senderEmail'])
            ->setFrom([Yii::$app->params['senderEmail'] => Yii::$app->params['senderName']])
            ->setSubject("New Dealer Registration : {$this->name} - " . Yii::$app->name)
            ->send();
    }
}
